#ifndef CREATURE_H
#define CREATURE_H

#include "RedBlack.h"
#include "IdSet.h"
#include "LnList.h"

#include "misc.h"
#include "Graphics.h"

class Dungeon;
class Object;

class Skill
{
	public:
		Skill( const char *, int );
		
		bool Test();
		bool operator == ( const char * );
		
	protected:
		friend class PlayerView;
		const char * name;
		int skill;
};

class SkillSet
{
	public:
		SkillSet();
		~SkillSet();
		
		void		Add( Skill * );
		Skill *		Find( const char * );
		Skill *		SkillAt( int );
		
	protected:
		List<Skill> skills;
};

class Creature;

class damage
{
	public:
		damage();
		
		int amount;
		int type;
		Creature * source;
		
		enum {
			SLASH,
			PIERCE,
			CRUSH,
			HEAT,
			COLD,
			ELECTRICITY
		};
};

class Creature
{ // base class for creatures (and player)
	public:
		enum {
			BACK		= 0x00000001,
			HEAD		= 0x00000002,
			LEFT_ARM	= 0x00000004,
			RIGHT_ARM	= 0x00000008,
			TORSO		= 0x00000010,
			LEFT_HAND	= 0x00000020,
			RIGHT_HAND	= 0x00000040,
			LEFT_LEG	= 0x00000080,
			RIGHT_LEG	= 0x00000100,
			LEFT_FOOT	= 0x00000200,
			RIGHT_FOOT	= 0x00000400
		};
		class Stat
		{
			public:
				int base;
				int curr;
		};
		
		Creature( const char *, Dungeon * );
		virtual ~Creature();

		virtual void AI(); // Let some AI have the critter do something
		virtual void Damage( damage );
		virtual void Death();
		virtual void Attack( Creature * );
		
		bool TestSkill( const char * );
		void DropObject( uint32 );
		bool PickUpObject( uint32 );
		
		void TurnToFace( Pos );
		
		Object * FindEquiped( uint32 ); // equip-mask

		IdSet		inventory; // all things carried
		IdSet		equipment; // currently equiped
		
		SkillSet	skills;
		Dungeon		* dungeon;
		
		uint32		id;
		uint32		bitmap;
		const char	* name;
		
		Pos			pos;
		
		Stat		INT, STR, HP;
		
		direction	facing;

	protected:
		bool FindPath( Pos );
		void FollowPath();
		bool HasPath();
		
	private:
		List<Pos>	* path;
};

class critter_info
{
	public:
		critter_info();
		
		const critter_info & operator = ( const critter_info & );
		bool operator != ( const critter_info & );
		bool operator < ( const critter_info & );
		
		Creature * creature;
};

class CreatureSet
{
	protected:
		uint32 next_critter_id;
		
		List<Creature> list;
//		RedBlack<critter_info>	list;
		
	public:
		CreatureSet();
		~CreatureSet();
		
		void		Add( Creature * );
		void		Remove( Creature * );
		Creature *	Find( uint32 );
		void		DoAI();
};

#endif
